from .admins import (add_nonadmin_chat, is_nonadmin_chat,
                     remove_nonadmin_chat)
from .auth import (_get_authusers, add_nonadmin_chat, delete_authuser,
                   get_authuser, get_authuser_count, get_authuser_names,
                   is_nonadmin_chat, remove_nonadmin_chat, save_authuser)
from .blacklistfilter import delete_blacklist_filter, get_blacklisted_words, save_blacklist_filter
from .blacklistuser import add_user_blacklist, is_user_blacklisted, remove_user_blacklist
from .gban import (add_gbanned, gbanned_users, is_gbanned,
                   remove_gbanned)
from .langs import get_lang, set_lang
from .pmpermit import (approve_pmpermit, disapprove_pmpermit,
                       is_pmpermit_approved)
from .premium import (add_prem, del_prem,
                      get_prem, is_prem)
from .start import _get_start, get_start, get_start_names, save_start
from .sudo import add_sudo, get_sudoers, remove_sudo
from .videocalls import (add_active_video_chat, get_active_video_chats,
                         get_video_limit, is_active_video_chat,
                         remove_active_video_chat, set_video_limit)
from .welcome import (
    captcha_off,
    captcha_on,
    del_welcome,
    get_captcha_cache,
    get_welcome,
    has_solved_captcha_once,
    is_captcha_on,
    save_captcha_solved,
    set_welcome,
    update_captcha_cache,
)



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport

from ._wrappers import eod, eor
from .func import Function
from .misc import Misc
from .pastebin import Paste
from .sections import section
from .toolbot import ToolBot
from .tools import Tools


class AyiinMethods(
    Function,
    Misc,
    Paste,
    ToolBot,
    Tools,
):
    pass



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import asyncio
import logging
import sys
import time
from aiohttp import ClientSession

from pyAyiin.Clients import *
from pyAyiin.config import *
from pyAyiin.methods import *
from pyAyiin.pyrogram import AyiinMethods
from pyAyiin.pyrogram import eod, eor
from pyAyiin.xd import GenSession
from pyAyiin.telethon.ayiin import *


# Bot Logs setup:
logging.basicConfig(
    format="[%(name)s] - [%(levelname)s] - %(message)s",
    level=logging.INFO,
)
logging.getLogger("asyncio").setLevel(logging.ERROR)
logging.getLogger("pyAyiin").setLevel(logging.INFO)
logging.getLogger("pytgcalls").setLevel(logging.INFO)
logging.getLogger("fipper").setLevel(logging.ERROR)
logging.getLogger("fipper.client").setLevel(logging.ERROR)
logging.getLogger("fipper.session.auth").setLevel(logging.ERROR)
logging.getLogger("fipper.session.session").setLevel(logging.ERROR)
logging.getLogger("pyrogram").setLevel(logging.ERROR)
logging.getLogger("pyrogram.client").setLevel(logging.ERROR)
logging.getLogger("pyrogram.session.auth").setLevel(logging.ERROR)
logging.getLogger("pyrogram.session.session").setLevel(logging.ERROR)
logging.getLogger("telethon").setLevel(logging.INFO)


logs = logging.getLogger(__name__)


__copyright__ = "Copyright (C) 2022-present AyiinXd <https://github.com/AyiinXd>"
__license__ = "GNU General Public License v3.0 (GPL-3.0)"
__version__ = "0.4.4.dev13"
ayiin_ver = "0.1.1"
prem_version = "0.1"


DEVS = [
    607067484, # Ayiin
    997461844, #Ayang_Ayiin
    2130526178, # Alfa
]

GCAST_BLACKLIST = [
    -1001797285258,  # AyiinChatsNew
    -1001675396283,  # AyiinXdSupport
    -1001473548283,  # SharingUserbot
    -1001433238829,  # TedeSupport
    -1001476936696,  # AnosSupport
    -1001327032795,  # UltroidSupport
    -1001294181499,  # UserBotIndo
    -1001419516987,  # VeezSupportGroup
    -1001459812644,  # GeezSupportGroup
    -1001296934585,  # X-PROJECT BOT
    -1001481357570,  # UsergeOnTopic
    -1001459701099,  # CatUserbotSupport
    -1001109837870,  # TelegramBotIndonesia
    -1001752592753,  # Skyzusupport
    -1001788983303,  # KayzuSupport
    -1001380293847,  # NastySupport
    -1001692751821,  # RamSupport
    -1001267233272,  # PocongUserbot
    -1001500063792,  # Trident
    -1001687155877,  # CilikSupport
    -1001578091827,  # PrimeSupport
    -1001704645461,  # Jamet No Support
    -1001662510083,  # MutualanDestra
    -1001347414136,  # ArunaMutualan
    -1001572486389,  # PluviaMusicGroup
    -1001608701614,  # UputtSupport
    -1001864253073,  # RitoSupport
]

StartTime = time.time()

var = Var()
HANDLER = f'{var.HNDLR}'
hndlr = (
    HANDLER.replace("[", "")
    .replace("'", "")
    .replace(",", "")
    .replace("]", "")
)


class PyrogramXd(AyiinMethods, GenSession, Methods):
    pass


class TelethonXd(AyiinMethod, GenSession, Methods):
    pass


suc_msg = (f"""
========================×========================
           Credit Py-Ayiin {__version__}
========================×========================
"""
)

fail_msg = (f"""
========================×========================
      Commit Yang Bener Bego Biar Gak Error
           Credit Py-Ayiin {__version__}
========================×========================
"""
)

start_bot = (f"""
========================×========================
         Starting AyiinUbot Version {ayiin_ver}
        Copyright (C) 2022-present AyiinXd
========================×========================
"""
)

run_as_module = False

if sys.argv[0] == "-m":
    run_as_module = True

    from .decorator import *
    from .config import *

    print("\n\n" + __copyright__ + "\n" + __license__)
    print(start_bot)

    CMD_HELP = {}
    adB = AyiinDB()
    aiosession = ClientSession()
    loop = asyncio.get_event_loop()
    HOSTED_ON = where_hosted()
    Yins = VcTools()
else:
    print(suc_msg)
    print(__copyright__ + "\n" + __license__ + "\n\n")

    aiosession = ClientSession()
    loop = asyncio.get_event_loop()
    HOSTED_ON = where_hosted()



# AyiinXd
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/AyiinXd >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/AyiinXd/blob/main/LICENSE/>.
#
# FROM AyiinXd <https://github.com/AyiinXd/AyiinXd>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

import aiohttp
import asyncio
import base64
import importlib
import math
import os
import shlex
import textwrap
from aiohttp import ContentTypeError
from typing import Tuple, Union
from io import BytesIO

from pymediainfo import MediaInfo
from fipper import Client, enums, raw
from fipper.types import Message, User
from PIL import Image, ImageDraw, ImageFont


_entitie = {
    raw.types.MessageEntityPhone: "phone_number",
    raw.types.MessageEntityMention: "mention",
    raw.types.MessageEntityBold: "bold",
    raw.types.MessageEntityCashtag: "cashtag",
    raw.types.MessageEntityStrike: "strikethrough",
    raw.types.MessageEntityHashtag: "hashtag",
    raw.types.MessageEntityEmail: "email",
    raw.types.MessageEntityMentionName: "text_mention",
    raw.types.MessageEntityUnderline: "underline",
    raw.types.MessageEntityUrl: "url",
    raw.types.MessageEntityTextUrl: "text_link",
    raw.types.MessageEntityBotCommand: "bot_command",
    raw.types.MessageEntityCode: "code",
    raw.types.MessageEntityPre: "pre",
}

class Function(object):
    def __init__(self):
        self._entities = _entitie
        self.o_api = "https://bot.lyo.su/quote/generate"
    
    
    async def get_readable_time(self, seconds: int) -> str:
        count = 0
        up_time = ""
        time_list = []
        time_suffix_list = ["s", "m", "Jam", "Hari"]
    
        while count < 4:
            count += 1
            remainder, result = divmod(
                seconds, 60) if count < 3 else divmod(
                seconds, 24)
            if seconds == 0 and remainder == 0:
                break
            time_list.append(int(result))
            seconds = int(remainder)
    
        for x in range(len(time_list)):
            time_list[x] = str(time_list[x]) + time_suffix_list[x]
        if len(time_list) == 4:
            up_time += time_list.pop() + ", "
    
        time_list.reverse()
        up_time += ":".join(time_list)
    
        return up_time
    
    
    async def add_text_img(
        self,
        image_path,
        text,
        fonts,
    ):
        font_size = 12
        stroke_width = 1
    
        if ";" in text:
            upper_text, lower_text = text.split(";")
        else:
            upper_text = text
            lower_text = ""
    
        img = Image.open(image_path).convert("RGBA")
        img_info = img.info
        image_width, image_height = img.size
        font = ImageFont.truetype(
            font=fonts,
            size=int(image_height * font_size) // 100,
        )
        draw = ImageDraw.Draw(img)
    
        char_width, char_height = font.getsize("A")
        chars_per_line = image_width // char_width
        top_lines = textwrap.wrap(upper_text, width=chars_per_line)
        bottom_lines = textwrap.wrap(lower_text, width=chars_per_line)
    
        if top_lines:
            y = 10
            for line in top_lines:
                line_width, line_height = font.getsize(line)
                x = (image_width - line_width) / 2
                draw.text(
                    (x, y),
                    line,
                    fill="white",
                    font=font,
                    stroke_width=stroke_width,
                    stroke_fill="black",
                )
                y += line_height
    
        if bottom_lines:
            y = image_height - char_height * len(bottom_lines) - 15
            for line in bottom_lines:
                line_width, line_height = font.getsize(line)
                x = (image_width - line_width) / 2
                draw.text(
                    (x, y),
                    line,
                    fill="white",
                    font=font,
                    stroke_width=stroke_width,
                    stroke_fill="black",
                )
                y += line_height
    
        final_image = os.path.join("memify.webp")
        img.save(final_image, **img_info)
        return final_image


# https://github.com/TeamUltroid/pyUltroid/blob/31c271cf4d35ab700e5880e952e54c82046812c2/pyUltroid/functions/helper.py#L154


    async def bash(self, cmd):
        process = await asyncio.create_subprocess_shell(
            cmd,
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        stdout, stderr = await process.communicate()
        err = stderr.decode().strip()
        out = stdout.decode().strip()
        return out, err
    
    
    async def run_cmd(self, cmd: str) -> Tuple[str, str, int, int]:
        args = shlex.split(cmd)
        process = await asyncio.create_subprocess_exec(
            *args, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )
        stdout, stderr = await process.communicate()
        return (
            stdout.decode("utf-8", "replace").strip(),
            stderr.decode("utf-8", "replace").strip(),
            process.returncode,
            process.pid,
        )
    
    
    # PyroHelper
    
    
    async def get_ub_chats(
        self,
        client,
        chat_types: list = [
            enums.ChatType.GROUP,
            enums.ChatType.SUPERGROUP,
            enums.ChatType.CHANNEL,
        ],
        is_id_only=True,
    ):
        ub_chats = []
        async for dialog in client.get_dialogs():
            if dialog.chat.type in chat_types:
                if is_id_only:
                    ub_chats.append(dialog.chat.id)
                else:
                    ub_chats.append(dialog.chat)
            else:
                continue
        return ub_chats
    
    
    def ReplyCheck(self, message: Message):
        reply_id = None
    
        if message.reply_to_message:
            reply_id = message.reply_to_message.id
    
        elif message.from_user.is_self:
            reply_id = message.id
    
        return reply_id
    
    
    def SpeedConvert(self, size):
        power = 2**10
        zero = 0
        units = {0: "", 1: "Kbit/s", 2: "Mbit/s", 3: "Gbit/s", 4: "Tbit/s"}
        while size > power:
            size /= power
            zero += 1
        return f"{round(size, 2)} {units[zero]}"
    
    
    def GetFromUserID(self, message: Message):
        """Get the user id of the incoming message."""
        return message.from_user.id
    
    
    def GetChatID(self, message: Message):
        """Get the group id of the incoming message"""
        return message.chat.id
    
    
    def GetUserMentionable(self, user: User):
        """Get mentionable text of a user."""
        if user.username:
            username = "@{}".format(user.username)
        else:
            if user.last_name:
                name_string = "{} {}".format(user.first_name, user.last_name)
            else:
                name_string = "{}".format(user.first_name)
    
            username = "<a href='tg://user?id={}'>{}</a>".format(
                user.id, name_string)
    
        return username
    
    
    async def async_searcher(
        self,
        url: str,
        post: bool = None,
        headers: dict = None,
        params: dict = None,
        json: dict = None,
        data: dict = None,
        ssl=None,
        re_json: bool = False,
        re_content: bool = False,
        real: bool = False,
    ):
        async with aiohttp.ClientSession(headers=headers) as client:
            if post:
                data = await client.post(url, json=json, data=data, ssl=ssl)
            else:
                data = await client.get(url, params=params, ssl=ssl)
            if re_json:
                return await data.json()
            if re_content:
                return await data.read()
            if real:
                return data
            return await data.text()
    
    
    async def _format_quote(self, message, reply=None, sender=None, type_="private"):
        from pyAyiin import DEVS
        
        
        async def telegraph(self, file_):
            file = f"{file_}.png"
            Image.open(file_).save(file, "PNG")
            files = {"file": open(file, "rb").read()}
            uri = (
                "https://telegra.ph"
                + (
                    await self.async_searcher(
                        "https://telegra.ph/upload", post=True, data=files, re_json=True
                    )
                )[0]["src"]
            )
            os.remove(file)
            os.remove(file_)
            return uri

        if reply:
            reply = {
                "name": reply.from_user.first_name or "Deleted Account",
                "text": reply.text,
                "chatId": reply.chat.id,
            }
        else:
            reply = {}
        is_fwd = message.from_user
        name = None
        last_name = None
        if sender and sender.id not in DEVS:
            id_ = sender.id
            name = sender.first_name
        elif not is_fwd:
            id_ = message.from_user.id
            sender = message.from_user
            name = message.from_user.first_name
        else:
            id_, sender = None, None
            name = is_fwd.first_name
            if is_fwd.id:
                try:
                    id_ = is_fwd.id
                    sender = await message._client.get_users(id_)
                    name = sender.first_name
                except ValueError:
                    pass
        if sender and hasattr(sender, "last_name"):
            last_name = sender.last_name
        entities = []
        if message.entities:
            for entity in message.entities:
                if type(entity) in self._entities:
                    enti_ = entity.to_dict()
                    del enti_["_"]
                    enti_["type"] = self._entities[type(entity)]
                    entities.append(enti_)
        message = {
            "entities": entities,
            "chatId": id_,
            "avatar": True,
            "from": {
                "id": id_,
                "first_name": (name or (sender.first_name if sender else None))
                or "Deleted Account",
                "last_name": last_name,
                "username": sender.username if sender else None,
                "language_code": "en",
                "title": name,
                "name": name or "Unknown",
                "type": type_,
            },
            "text": message.text,
            "replyMessage": {},
        }

        return message


    async def create_quotly(
        self,
        message,
        url="https://qoute-api-akashpattnaik.koyeb.app/generate",
        reply={},
        bg=None,
        sender=None,
        file_name="quote.webp",
    ):
        if not isinstance(message, list):
            message = [message]
            url = self.o_api
        if not bg:
            bg = "#1b1429"
        content = {
            "type": "quote",
            "format": "webp",
            "backgroundColor": bg,
            "width": 512,
            "height": 768,
            "scale": 2,
            "messages": [
                await self._format_quote(msg, reply=reply, sender=sender)
                for msg in message
            ],
        }
        try:
            request = await self.async_searcher(url, post=True, json=content, re_json=True)
        except ContentTypeError as er:
            if url != self.o_api:
                return await self.create_quotly(
                    message,
                    url=self.o_api,
                    bg=bg,
                    sender=sender,
                    reply=reply,
                    file_name=file_name,
                )
            raise er
        if request.get("ok"):
            with open(file_name, "wb") as file:
                image = base64.decodebytes(
                    request["result"]["image"].encode("utf-8"))
                file.write(image)
            return file_name
        raise Exception(str(request))
    
    
    async def Carbon(
        self,
        code,
        base_url="https://carbonara-42.herokuapp.com/api/cook",
        file_name="Ayiin-Ubot",
        **kwargs,
    ):
        kwargs["code"] = code
        con = await self.async_searcher(base_url, post=True, json=kwargs, re_content=True)
        file = BytesIO(con)
        file.name = f"{file_name}.jpg"
        return file
    
    
    def humanbytes(self, size: Union[int, float]) -> str:
        if size is None or isinstance(size, str):
            return ""
    
        power = 2**10
        raised_to_pow = 0
        dict_power_n = {0: "", 1: "Ki", 2: "Mi", 3: "Gi", 4: "Ti"}
        while size > power:
            size /= power
            raised_to_pow += 1
        return f"{str(round(size, 2))} {dict_power_n[raised_to_pow]}B"



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

from .client import *
from .pytgcalls import *
from .startup import StartPyrogram, clients, client_id


class VcTools(
    #ClientCalls,
    GroupCalls,
):
    pass



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/TeamUltroid/pyUltroid/blob/main/LICENSE/>.
#
# Ported by @mrismanaziz
# FROM Man-Userbot <https://github.com/mrismanaziz/Man-Userbot>
# t.me/SharingUserbot & t.me/Lunatic0de

import base64
import os
import os.path
from io import BytesIO

import aiohttp
from aiohttp import ContentTypeError
from PIL import Image
from telethon.tl import types
from telethon.utils import get_display_name, get_peer_id

DEVS = [1905050903, 1965424892]


_entities = {
    types.MessageEntityPhone: "phone_number",
    types.MessageEntityMention: "mention",
    types.MessageEntityBold: "bold",
    types.MessageEntityCashtag: "cashtag",
    types.MessageEntityStrike: "strikethrough",
    types.MessageEntityHashtag: "hashtag",
    types.MessageEntityEmail: "email",
    types.MessageEntityMentionName: "text_mention",
    types.MessageEntityUnderline: "underline",
    types.MessageEntityUrl: "url",
    types.MessageEntityTextUrl: "text_link",
    types.MessageEntityBotCommand: "bot_command",
    types.MessageEntityCode: "code",
    types.MessageEntityPre: "pre",
}


class Misc_(object):
    async def _format_quote(
        self,
        event,
        reply=None,
        sender=None,
        type_="private"
    ):
        async def telegraph(self, file_):
            file = f"{file_}.png"
            Image.open(file_).save(file, "PNG")
            files = {"file": open(file, "rb").read()}
            uri = (
                "https://telegra.ph"
                + (
                    await self.async_searcher(
                        "https://telegra.ph/upload", post=True, data=files, re_json=True
                    )
                )[0]["src"]
            )
            os.remove(file)
            os.remove(file_)
            return uri

        if reply:
            reply = {
                "name": get_display_name(reply.sender) or "Deleted Account",
                "text": reply.raw_text,
                "chatId": reply.chat_id,
            }
        else:
            reply = {}
        is_fwd = event.fwd_from
        name = None
        last_name = None
        if sender and sender.id not in DEVS:
            id_ = get_peer_id(sender)
            name = get_display_name(sender)
        elif not is_fwd:
            id_ = event.sender_id
            sender = await event.get_sender()
            name = get_display_name(sender)
        else:
            id_, sender = None, None
            name = is_fwd.from_name
            if is_fwd.from_id:
                id_ = get_peer_id(is_fwd.from_id)
                try:
                    sender = await event.client.get_entity(id_)
                    name = get_display_name(sender)
                except ValueError:
                    pass
        if sender and hasattr(sender, "last_name"):
            last_name = sender.last_name
        entities = []
        if event.entities:
            for entity in event.entities:
                if type(entity) in _entities:
                    enti_ = entity.to_dict()
                    del enti_["_"]
                    enti_["type"] = _entities[type(entity)]
                    entities.append(enti_)
        message = {
            "entities": entities,
            "chatId": id_,
            "avatar": True,
            "from": {
                "id": id_,
                "first_name": (name or (sender.first_name if sender else None))
                or "Deleted Account",
                "last_name": last_name,
                "username": sender.username if sender else None,
                "language_code": "en",
                "title": name,
                "name": name or "Unknown",
                "type": type_,
            },
            "text": event.raw_text,
            "replyMessage": reply,
        }
        if event.document and event.document.thumbs:
            file_ = await event.download_media(thumb=-1)
            uri = await self.telegraph(file_)
            message["media"] = {"url": uri}

        return message


    async def create_quotly(
        self,
        event,
        url="https://qoute-api-akashpattnaik.koyeb.app/generate",
        reply={},
        bg=None,
        sender=None,
        file_name="quote.webp",
    ):
        O_API = "https://bot.lyo.su/quote/generate"
        if not isinstance(event, list):
            event = [event]
            url = O_API
        if not bg:
            bg = "#1b1429"
        content = {
            "type": "quote",
            "format": "webp",
            "backgroundColor": bg,
            "width": 512,
            "height": 768,
            "scale": 2,
            "messages": [
                await self._format_quote(message, reply=reply, sender=sender)
                for message in event
            ],
        }
        try:
            request = await self.async_searcher(url, post=True, json=content, re_json=True)
        except ContentTypeError as er:
            if url != O_API:
                return await self.create_quotly(event,
                        url=O_API,
                        bg=bg,
                        sender=sender,
                        reply=reply,
                        file_name=file_name,
                )
            raise er
        if request.get("ok"):
            with open(file_name, "wb") as file:
                image = base64.decodebytes(request["result"]["image"].encode("utf-8"))
                file.write(image)
            return file_name
        raise Exception(str(request))


    async def Carbon(
        self,
        code,
        base_url="https://carbonara-42.herokuapp.com/api/cook",
        file_name="Ayiin-Userbot",
        **kwargs,
    ):
        kwargs["code"] = code
        con = await self.async_searcher(base_url, post=True, json=kwargs, re_content=True)
        file = BytesIO(con)
        file.name = f"{file_name}.jpg"
        return file


    async def animator(
        self,
        media,
        mainevent,
        textevent
    ):
        # Coded by @Jisan7509
        h = media.file.height
        w = media.file.width
        w, h = (-1, 512) if h > w else (512, -1)
        if not os.path.isdir(self.TEMP_DOWNLOAD_DIRECTORY):
            os.makedirs(self.TEMP_DOWNLOAD_DIRECTORY)
        Ayiin = await mainevent.client.download_media(media, self.TEMP_DOWNLOAD_DIRECTORY)
        await textevent.edit("`Converting...`")
        await self.run_cmd(
            f"ffmpeg -ss 00:00:00 -to 00:00:02.900 -i {Ayiin} -vf scale={w}:{h} -c:v libvpx-vp9 -crf 30 -b:v 560k -maxrate 560k -bufsize 256k -an Video.webm"
        )
        os.remove(Ayiin)
        vid = "Video.webm"
        return vid




from ._core import mongodb

videodb = mongodb.Ayiinvideocalls


# limit


async def get_video_limit(chat_id: int) -> str:
    limit = await videodb.find_one({"chat_id": chat_id})
    if not limit:
        return ""
    return limit["limit"]


async def set_video_limit(chat_id: int, limit: str):
    return await videodb.update_one(
        {"chat_id": chat_id}, {"$set": {"limit": limit}}, upsert=True
    )


# Queue Chats Video


async def get_active_video_chats() -> list:
    chats = videodb.find({"chat_id": {"$lt": 0}})
    if not chats:
        return []
    chats_list = []
    for chat in await chats.to_list(length=1000000000):
        chats_list.append(chat)
    return chats_list


async def is_active_video_chat(chat_id: int) -> bool:
    chat = await videodb.find_one({"chat_id": chat_id})
    if not chat:
        return False
    return True


async def add_active_video_chat(chat_id: int):
    is_served = await is_active_video_chat(chat_id)
    if is_served:
        return
    return await videodb.insert_one({"chat_id": chat_id})


async def remove_active_video_chat(chat_id: int):
    is_served = await is_active_video_chat(chat_id)
    if not is_served:
        return
    return await videodb.delete_one({"chat_id": chat_id})



from ._core import mongodb


authdb = mongodb.admins

nonadmin = {}


async def is_nonadmin_chat(chat_id: int) -> bool:
    mode = nonadmin.get(chat_id)
    if not mode:
        user = await authdb.find_one({"chat_id": chat_id})
        if not user:
            nonadmin[chat_id] = False
            return False
        nonadmin[chat_id] = True
        return True
    return mode


async def add_nonadmin_chat(chat_id: int):
    nonadmin[chat_id] = True
    is_admin = await check_nonadmin_chat(chat_id)
    if is_admin:
        return
    return await authdb.insert_one({"chat_id": chat_id})


async def remove_nonadmin_chat(chat_id: int):
    nonadmin[chat_id] = False
    is_admin = await check_nonadmin_chat(chat_id)
    if not is_admin:
        return
    return await authdb.delete_one({"chat_id": chat_id})




from ._core import mongodb

sudoersdb = mongodb.sudoers


async def get_sudoers() -> list:
    sudoers = await sudoersdb.find_one({"sudo": "sudo"})
    if not sudoers:
        return []
    return sudoers["sudoers"]


async def add_sudo(user_id: int) -> bool:
    sudoers = await get_sudoers()
    sudoers.append(user_id)
    await sudoersdb.update_one(
        {"sudo": "sudo"}, {"$set": {"sudoers": sudoers}}, upsert=True
    )
    return True


async def remove_sudo(user_id: int) -> bool:
    sudoers = await get_sudoers()
    sudoers.remove(user_id)
    await sudoersdb.update_one(
        {"sudo": "sudo"}, {"$set": {"sudoers": sudoers}}, upsert=True
    )
    return True


async def check_sudo(sudo_id):
    already_sudo = await sudoersdb.find_one({"sudo": "sudo"})
    if already_sudo:
        sudo_list = [int(sudo_id) for sudo_id in already_sudo.get("sudo_id")]
        if int(sudo_id) in sudo_list:
            return True
        else:
            return False
    else:
        return False



from ._core import mongodb

pmpermitdb = mongodb.permit


BLOCK_MSG = "Sorry tod lu di blokir karna melakukan spam!!!"

LIMIT = 5

MSG_PERMIT = (
    """
╔═════════════════════╗
│  𖣘 𝚂𝙴𝙻𝙰𝙼𝙰𝚃 𝙳𝙰𝚃𝙰𝙽𝙶 𝚃𝙾𝙳 𖣘ㅤ  ㅤ
╚═════════════════════╝
 ⍟ 𝙹𝙰𝙽𝙶𝙰𝙽 𝚂𝙿𝙰𝙼 𝙲𝙷𝙰𝚃 𝙼𝙰𝙹𝙸𝙺𝙰𝙽 𝙶𝚄𝙰 𝙺𝙴𝙽𝚃𝙾𝙳
 ⍟ 𝙶𝚄𝙰 𝙰𝙺𝙰𝙽 𝙾𝚃𝙾𝙼𝙰𝚃𝙸𝚂 𝙱𝙻𝙾𝙺𝙸𝚁 𝙺𝙰𝙻𝙾 𝙻𝚄 𝚂𝙿𝙰𝙼
 ⍟ 𝙹𝙰𝙳𝙸 𝚃𝚄𝙽𝙶𝙶𝚄 𝚂𝙰𝙼𝙿𝙰𝙸 𝙼𝙰𝙹𝙸𝙺𝙰𝙽 𝙶𝚄𝙰 𝙽𝙴𝚁𝙸𝙼𝙰 𝙿𝙴𝚂𝙰𝙽 𝙻𝚄
╔═════════════════════╗
│ㅤㅤ𖣘 𝙿𝙴𝚂𝙰𝙽 𝙾𝚃𝙾𝙼𝙰𝚃𝙸𝚂 𖣘ㅤㅤ
│ㅤㅤ   𖣘 𝙰𝚈𝙸𝙸𝙽 - 𝚄𝙱𝙾𝚃 𖣘ㅤㅤ
╚═════════════════════╝
"""
)



async def is_pmpermit_approved(user_id: int, chat_id: int) -> bool:
    user = await pmpermitdb.find_one({"user_id": user_id, "chat_id": chat_id})
    if not user:
        return False
    return True


async def approve_pmpermit(user_id: int, chat_id: int):
    is_pmpermit = await is_pmpermit_approved(user_id, chat_id)
    if is_pmpermit:
        return
    return await pmpermitdb.insert_one({"user_id": user_id, "chat_id": chat_id})


async def disapprove_pmpermit(user_id: int, chat_id: int):
    is_pmpermit = await is_pmpermit_approved(user_id, chat_id)
    if not is_pmpermit:
        return
    return await pmpermitdb.delete_one({"user_id": user_id, "chat_id": chat_id})


async def set_pmpermit(user_id, value: bool):
    await pmpermitdb.update_one(
        {"user_id": user_id},
        {"$set": {"pmpermit": value}},
        upsert=True
    )


async def get_pmpermit(user_id: int):
    x = await pmpermitdb.find_one({"user_id": user_id})
    if not x:
        return False
    return x


async def message_pmpermit(user_id: int, text: str):
    doc = {"user_id": user_id, "pmpermit_message": text}
    await pmpermitdb.update_one(
        {"user_id": user_id},
        {"$set": doc},
        upsert=True)


async def get_message_pmpermit(user_id: int):
    x = await get_pmpermit(user_id)
    if x["pmpermit_message"] is not None:
        return x["pmpermit_message"]
    else:
        return MSG_PERMIT


async def limit_pmpermit(user_id: int, limit):
    doc = {"user_id": user_id, "limit": limit}
    await pmpermitdb.update_one(
      {"user_id": user_id},
      {"$set": doc},
      upsert=True)


async def get_limit_pmpermit(user_id: int):
    x = await get_pmpermit(user_id)
    if x["limit"] is not None:
        return x["limit"]
    else:
        return LIMIT


async def block_message_pmpermit(user_id: int, text: str):
    doc = {"user_id": user_id, "block_message": text}
    await pmpermitdb.update_one(
      {"user_id": user_id},
      {"$set": doc},
      upsert=True)


async def get_block_pmpermit(user_id: int):
    x = await get_pmpermit(user_id)
    if x["block_message"] is not None:
        return x["block_message"]
    else:
        return BLOCK_MSG


async def media_pmpermit(user_id: int, url: str):
    doc = {"user_id": user_id, "media_message": url}
    await pmpermitdb.update_one(
      {"user_id": user_id},
      {"$set": doc},
      upsert=True)


async def get_media_pmpermit(user_id: int):
    x = await get_pmpermit(user_id)
    if x["media_message"] is not None:
        return x["media_message"]
    else:
        return None


async def all_pmpermit(user_id: int):
    result = await pmpermitdb.find_one({"user_id": user_id})
    #if not result:
    #    return MSG_PERMIT, None, LIMIT
    permit = result["pmpermit"]
    pm_message = result.get("pmpermit_message") if result.get("pmpermit_message") is not None else MSG_PERMIT
    media_message = result.get("media_message") 
    limit = result.get("limit") if result.get("limit") is not None else LIMIT
    return pm_message, media_message, limit




'''
from ._core import mongodb

pmpermitdb = mongodb.permit


async def is_pmpermit_approved(user_id: int) -> bool:
    user = await pmpermitdb.find_one({"user_id": user_id})
    if not user:
        return False
    return True


async def approve_pmpermit(user_id: int):
    is_pmpermit = await is_pmpermit_approved(user_id)
    if is_pmpermit:
        return
    return await pmpermitdb.insert_one({"user_id": user_id})


async def disapprove_pmpermit(user_id: int):
    is_pmpermit = await is_pmpermit_approved(user_id)
    if not is_pmpermit:
        return
    return await pmpermitdb.delete_one({"user_id": user_id})
'''


# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

import asyncio
import os
import yt_dlp

from typing import Union

from .changer import Changers

try:
    from youtubesearchpython import VideosSearch
except ImportError:
    print("'youtube-search-python' tidak terinstall\nmungkin beberapa modul tidak akan berjalan")
    VideosSearch = None


yins = Changers()
base = "https://www.youtube.com/watch?v="


class Funci(object):
    def yt_info_query(self, query: str):
        results = VideosSearch(query, limit=1)
        for result in results.result()["result"]:
            title = result["title"]
            duration_min = result["duration"]
            thumbnail = result["thumbnails"][0]["url"].split("?")[0]
            videoid = result["id"]
            if str(duration_min) == "None":
                duration_sec = 0
            else:
                duration_sec = int(yins.time_to_seconds(duration_min))
        return title, duration_min, duration_sec, thumbnail, videoid
    
    def yt_info_id(self, videoid):
        url = f"https://www.youtube.com/watch?v={videoid}"
        results = VideosSearch(url, limit=1)
        for result in results.result()["result"]:
            title = result["title"]
            duration_min = result["duration"]
            thumbnail = result["thumbnails"][0]["url"].split("?")[0]
            if str(duration_min) == "None":
                duration_sec = 0
            else:
                duration_sec = int(yins.time_to_seconds(duration_min))
        return title, duration_min, duration_sec, thumbnail
    
    def yt_info_query_slider(self, query: str, query_type: int):
        a = VideosSearch(query, limit=10)
        result = (a.result()).get("result")
        title = result[query_type]["title"]
        duration_min = result[query_type]["duration"]
        videoid = result[query_type]["id"]
        thumbnail = result[query_type]["thumbnails"][0]["url"].split("?")[0]
        if str(duration_min) == "None":
            duration_sec = 0
        else:
            duration_sec = int(yins.time_to_seconds(duration_min))
        return title, duration_min, duration_sec, thumbnail, videoid


    async def get_m3u8(self, videoid):
        link = f"https://www.youtube.com/watch?v={videoid}"
        proc = await asyncio.create_subprocess_exec(
            "yt-dlp",
            "-g",
            "-f",
            "best[height<=?720][width<=?1280]",
            f"{link}",
            stdout=asyncio.subprocess.PIPE,
            stderr=asyncio.subprocess.PIPE,
        )
        stdout, stderr = await proc.communicate()
        if stdout:
            return 1, stdout.decode().split("\n")[0]
        else:
            return 0, stderr.decode()


    async def download(
        self,
        link: str,
        video: Union[bool, str] = None,
        videoid: Union[bool, str] = None,
        songaudio: Union[bool, str] = None,
        songvideo: Union[bool, str] = None,
        format_id: Union[bool, str] = None,
        title: Union[bool, str] = None,
    ) -> str:
        if videoid:
            link = base + link
        loop = asyncio.get_running_loop()

        def audio_dl():
            ydl_optssx = {
                "format": "bestaudio/best",
                "outtmpl": "downloads/%(id)s.%(ext)s",
                "geo_bypass": True,
                "nocheckcertificate": True,
                "quiet": True,
                "no_warnings": True,
            }
            x = yt_dlp.YoutubeDL(ydl_optssx)
            info = x.extract_info(link, False)
            xyz = os.path.join(
                "downloads", f"{info['id']}.{info['ext']}"
            )
            if os.path.exists(xyz):
                return xyz
            x.download([link])
            return xyz

        def video_dl():
            ydl_optssx = {
                "format": "(bestvideo[height<=?720][width<=?1280][ext=mp4])+(bestaudio[ext=m4a])",
                "outtmpl": "downloads/%(id)s.%(ext)s",
                "geo_bypass": True,
                "nocheckcertificate": True,
                "quiet": True,
                "no_warnings": True,
            }
            x = yt_dlp.YoutubeDL(ydl_optssx)
            info = x.extract_info(link, False)
            xyz = os.path.join(
                "downloads", f"{info['id']}.{info['ext']}"
            )
            if os.path.exists(xyz):
                return xyz
            x.download([link])
            return xyz

        def song_video_dl():
            formats = f"{format_id}+140"
            fpath = f"downloads/{title}"
            ydl_optssx = {
                "format": formats,
                "outtmpl": fpath,
                "geo_bypass": True,
                "nocheckcertificate": True,
                "quiet": True,
                "no_warnings": True,
                "prefer_ffmpeg": True,
                "merge_output_format": "mp4",
            }
            x = yt_dlp.YoutubeDL(ydl_optssx)
            x.download([link])

        def song_audio_dl():
            fpath = f"downloads/{title}.%(ext)s"
            ydl_optssx = {
                "format": format_id,
                "outtmpl": fpath,
                "geo_bypass": True,
                "nocheckcertificate": True,
                "quiet": True,
                "no_warnings": True,
                "prefer_ffmpeg": True,
                "postprocessors": [
                    {
                        "key": "FFmpegExtractAudio",
                        "preferredcodec": "mp3",
                        "preferredquality": "192",
                    }
                ],
            }
            x = yt_dlp.YoutubeDL(ydl_optssx)
            x.download([link])

        if songvideo:
            await loop.run_in_executor(None, song_video_dl)
            fpath = f"downloads/{title}.mp4"
            return fpath
        elif songaudio:
            await loop.run_in_executor(None, song_audio_dl)
            fpath = f"downloads/{title}.mp3"
            return fpath
        elif video:
            proc = await asyncio.create_subprocess_exec(
                "yt-dlp",
                "-g",
                "-f",
                "best[height<=?720][width<=?1280]",
                f"{link}",
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            stdout, stderr = await proc.communicate()
            if stdout:
                downloaded_file = stdout.decode().split("\n")[0]
                direct = None
            else:
                return
        else:
            direct = True
            downloaded_file = await loop.run_in_executor(
                None, audio_dl
            )
        return downloaded_file, direct



from ._core import mongodb

langdb = mongodb.langs

langm = {}


# language
async def get_lang(user_id: int) -> str:
    mode = langm.get(user_id)
    if not mode:
        lang = await langdb.find_one({"user_id": user_id})
        if not lang:
            langm[user_id] = "en"
            return "en"
        langm[user_id] = lang["lang"]
        return lang["lang"]
    return mode


async def set_lang(user_id: int, lang: str):
    langm[user_id] = lang
    await langdb.update_one(
        {"user_id": user_id}, {"$set": {"lang": lang}}, upsert=True
    )



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

from ._database import AyiinDB
from ._misc import _Misc
from .changer import Changers
from .converter import Convert
from .func import Funci
from .funcb import FuncBot
from .helpers import Helpers, run_async
from .hosting import where_hosted
from .inlinebot import InlineBot
from .queue import Queues
from .thumbnail import Thumbnail


class Methods(
    _Misc,
    Changers,
    Convert,
    Funci,
    FuncBot,
    InlineBot,
    Helpers,
    Queues,
    Thumbnail,
):
    pass
